SystemOrganization addCategory: #NewspeakCore!

nil subclass: #ImplementationBase
	instanceVariableNames: ''
	classVariableNames: 'TheSqueakVmMirror'
	poolDictionaries: ''
	category: 'NewspeakCore'!

!ImplementationBase commentStamp: '<historical>' prior: 0!
Class ImplementationBase provides transitional or platform specific features required by the underlying platform to interact with Newspeak objects. Newspeak programs should not rely on any behavior provided by this class as it is subject to removal at any time.

   Copyright 2008 Cadence Design Systems, Inc.
   
   Licensed under the Apache License, Version 2.0 (the ''License''); you may not use this file except in compliance with the License.  You may obtain a copy of the License at  http://www.apache.org/licenses/LICENSE-2.0!

!ImplementationBase class methodsFor: 'as yet unclassified' stamp: 'ryan 6/29/2011 08:57'!
initialize
	"Guard against Behavior>>initialize"
	
	self methodDictionary at: #'initializer`new' put: CompiledMethod toReturnSelf. "Faster for a stub to be there than for it to be rerouted to new"! !

!ImplementationBase class methodsFor: 'as yet unclassified' stamp: 'Ahe 9/5/2008 09:53'!
initializeSlotNamed: slotName "<String>" to: obj "<Object>"
	| ou |
	ou := self vmMirror objectUtil.
	self allSubInstancesDo: [ :instance |
		ou slotNamed: slotName of: instance put: obj]! !

!ImplementationBase class methodsFor: 'as yet unclassified' stamp: 'ryan 6/21/2011 09:24'!
new
	^self basicNew! !

!ImplementationBase class methodsFor: 'as yet unclassified' stamp: 'Ahe 9/21/2008 22:54'!
newInstanceFrom: oldInstance variable: variable size: instSize map: map
	| newInstance objectUtil |
	objectUtil := self vmMirror objectUtil.
	newInstance := variable
		ifTrue: [self basicNew: (objectUtil indexedSlotsOf: oldInstance)]
		ifFalse: [self basicNew].
	objectUtil namedSlotIndicesOf: newInstance do: [ :i | | oldOffset obj |
		oldOffset := map at: i.
		oldOffset == 0 ifFalse: [
			obj := objectUtil namedSlotOf: oldInstance at: oldOffset.
			objectUtil namedSlotOf: newInstance at: i put: obj]].
	variable ifTrue: [
		objectUtil indexedSlotIndicesOf: newInstance do: [ :i | | obj |
			obj := objectUtil indexedSlotOf: oldInstance at: i.
			objectUtil indexedSlotOf: newInstance at: i put: obj]].
	oldInstance isImmutable ifTrue: [newInstance isImmutable: true].
	^newInstance! !

!ImplementationBase class methodsFor: 'as yet unclassified' stamp: 'ryan 3/13/2012 15:13'!
vmMirror
	#BOGUS yourself. "Leaks capability"
	nil == TheSqueakVmMirror ifTrue: [
		TheSqueakVmMirror := SqueakVmMirror usingPlatform: BlackMarket platform].
	^TheSqueakVmMirror! !

!ImplementationBase methodsFor: 'squeak-inspecting' stamp: 'ryan 6/21/2011 10:30'!
asSetElement
	^self! !

!ImplementationBase methodsFor: 'squeak-inspecting' stamp: 'ryan 6/21/2011 09:05'!
basicSize
	"Primitive. Answer the number of indexable variables in the receiver. 
	This value is the same as the largest legal subscript. Essential. Do not 
	override in any subclass. See Object documentation whatIsAPrimitive."

	<primitive: 62>
	"The number of indexable fields of fixed-length objects is 0"
	^0! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'Ahe 8/27/2008 14:20'!
cannotInterpret: message "<Message>"

	Error signal: 'Cannot interpret'.
	^message sentTo: self "retry in case the problem was resolved"! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'ryan 11/23/2011 15:09'!
clone

	<primitive: 148>
	self primitiveFailed! !

!ImplementationBase methodsFor: 'squeak-inspecting' stamp: 'GB 2/25/2010 10:31'!
defaultLabelForInspector
	"Obsolete Squeakism"
	"Used by Squeak inspector - only here for Squeak IDE integration"
	"Answer the default label to be used for an Inspector window on the receiver."
	#BOGUS yourself.
	^ self class name! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'ryan 6/21/2011 09:04'!
doesNotUnderstand: aMessage

	^ MessageNotUnderstood new 
		message: aMessage;
		receiver: self;
		signal! !

!ImplementationBase methodsFor: 'squeak-inspecting' stamp: 'ryan 6/21/2011 10:30'!
enclosedSetElement
	^self! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'Ahe 8/28/2008 12:44'!
explore
	"Obsolete Squeakism"
	ObsoletionHelper warnObsolete.
	self inspect! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'GB 7/2/2009 13:36'!
flag: ignored "<Symbol>"
	"ObsoletionHelper warnObsolete"! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'Ahe 2/15/2009 20:50'!
halt
	"Halts execution at the sender of this message."
	ObsoletionHelper warnObsolete.
	Halt signal! !

!ImplementationBase methodsFor: 'squeak-inspecting' stamp: 'ryan 10/10/2011 16:30'!
iconOrThumbnailOfSize: aNumberOrPoint
	"Answer an appropiate form to represent the receiver"
	^ nil! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'Ahe 9/3/2008 19:18'!
ifNil: action "<[]>"
	^self! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'ryan 11/7/2010 16:53'!
ifNil: nilBlock ifNotNil: ifNotNilBlock
	"Evaluate the block, unless I'm == nil (q.v.)"

	^ ifNotNilBlock valueWithPossibleArgument: self! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'Ahe 9/3/2008 19:18'!
ifNotNil: action "<[] | [:self]>"
	^action valueWithPossibleArgument: self! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'ryan 5/1/2011 11:06'!
ifNotNil: ifNotNilBlock ifNil: nilBlock
	"Evaluate the block, unless I'm == nil (q.v.)"

	^ ifNotNilBlock valueWithPossibleArgument: self! !

!ImplementationBase methodsFor: 'squeak-inspecting' stamp: 'Unknown 8/6/2012 13:48'!
inboundPointersExcluding: objectsToExclude
	"Answer a list of all objects in the system that point to me, excluding those in the collection of objectsToExclude. I do my best to avoid creating any temporary objects that point to myself, especially method and block contexts. Adapted from PointerFinder class >> #pointersTo:except:"
	| anObj pointers objectsToAlwaysExclude |
	Smalltalk garbageCollect.
	"big collection shouldn't grow, so it's contents array is always the same"
	pointers := OrderedCollection new: 1000.
	
	"#allObjectsDo: and #pointsTo: are expanded inline to keep spurious	 method and block contexts out of the results"
	anObj := nil someObject.	
	[0 == anObj] whileFalse: [ "We must use #== here, to avoid leaving the loop when anObj is another number that's equal to 0 (e.g. 0.0)."
		anObj isInMemory
			ifTrue: [((anObj instVarsInclude: self)
				or: [anObj class == self])
					ifTrue: [pointers add: anObj]].
		anObj := anObj nextObject].
	
	objectsToAlwaysExclude := {
		pointers collector.
		thisContext.
		thisContext sender.
		thisContext sender sender.
		objectsToExclude.
	}.	
		
	^ pointers removeAllSuchThat: [:ea |
		(objectsToAlwaysExclude identityIncludes: ea)
			or: [objectsToExclude identityIncludes: ea]]! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'ryan 2/22/2012 12:11'!
inspect
	"Obsolete Squeakism"
	ObsoletionHelper warnObsolete.
	BlackMarket ide IDEWindow openOnObject: self! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'Ahe 8/28/2008 12:44'!
inspectorClass
	"Obsolete Squeakism"
	"Used by debugger"
	ObsoletionHelper warnObsolete.
	^SqueakVmMirrorInspector! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'GB 10/3/2009 08:55'!
instVarAt: index 
	"Primitive. Answer a fixed variable in an object. The numbering of the 
	variables corresponds to the named instance variables. Fail if the index 
	is not an Integer or is not the index of a fixed variable. Essential. See 
	Object documentation whatIsAPrimitive."

	<primitive: 73>
	"Access beyond fixed variables."
	^self basicAt: index - self class instSize		! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'yk 10/19/2011 09:39'!
instVarsInclude: anObject
"Answers true if anObject is among my named or indexed instance variables, and false otherwise"

	<primitive: 132>
	1 to: self class instSize do:
		[:i | (self instVarAt: i) == anObject ifTrue: [^ true]].
	1 to: self basicSize do:
		[:i | (self basicAt: i) == anObject ifTrue: [^ true]].
	^ false! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'ryan 6/7/2011 21:45'!
interestingContextOf: context "<ContextPart>"
	| interestingContext |
	interestingContext := context.
	[interestingContext selector includes: Language syntheticNameSeparatorCharacter]
		whileTrue: [interestingContext := interestingContext sender].
	^interestingContext! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'Unknown 8/6/2012 13:54'!
isAppObject
	^false! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'Ahe 8/27/2008 14:22'!
isBlock
	^false! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'Ahe 9/2/2008 17:03'!
isImmutable: aBoolean
	"Obsolete: should be controlled by a capability"
	"Primitive.  Determine whether an object is immutable
	 (true means immutable) or mutable (false means mutable).
	 Fail if the receiver is an immediate (SmallInteger).
	 Essential. See Object documentation whatIsAPrimitive."

	<primitive: 164>
	self Error signalPrimitiveFailed.
	^nil! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'Ahe 8/27/2008 14:22'!
isInMemory
	"Squeak crap: pointer finder"
	^true! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'rm 6/20/2009 13:07'!
isKindOf: cls
	#BOGUS. "rm: sorry, had to block transcript on this for new ide window to open"
	"ObsoletionHelper warnObsolete."
	^(self class == cls) or: [self class inheritsFrom: cls]! !

!ImplementationBase methodsFor: 'squeak-inspecting' stamp: 'ryan 6/21/2011 09:05'!
isLiteral
	^false! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'Ahe 8/27/2008 14:22'!
isLiteralUsing: ignored
	"Used by PointerFinder"
	^false! !

!ImplementationBase methodsFor: 'squeak-inspecting' stamp: 'ryan 6/21/2011 09:12'!
isMemberOf: aClass 
	"Answer whether the receiver is an instance of the class, aClass."

	^self class == aClass! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'Ahe 8/28/2008 12:45'!
isSelfEvaluating
	ObsoletionHelper warnObsolete.
	^false! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'Ahe 8/28/2008 12:47'!
isSelfEvaluatingUsing: ignored
	^false! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'ryan 6/10/2012 09:47'!
longPrintOn: aStream
	"Append to the argument, aStream, the names and values of all 
	of the receiver's instance variables."

	self flag: 'reimplement with mirrors?'.
	self class allInstVarNames doWithIndex:
		[:title :index |
		aStream nextPutAll: title;
		 nextPut: $:;
		 space;
		 tab;
		 print: (self instVarAt: index);
		 cr]! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'ryan 6/10/2012 09:47'!
longPrintOn: aStream limitedTo: sizeLimit indent: indent
	"Append to the argument, aStream, the names and values of all of the receiver's instance variables.  Limit is the length limit for each inst var."

	self flag: 'reimplement with mirrors?'.
	self class allInstVarNames doWithIndex:
		[:title :index |
		indent timesRepeat: [aStream tab].
		aStream nextPutAll: title;
		 nextPut: $:;
		 space;
		 tab;
		 nextPutAll: 
			((self instVarAt: index) printStringLimitedTo: (sizeLimit -3 -title size max: 1));
		 cr]! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'Ahe 8/27/2008 14:23'!
nextInstance
	<primitive: 78>
	^nil! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'Unknown 8/6/2012 13:51'!
nextObject
	"Primitive. Answer the next object after the receiver in the
	enumeration of all objects. Return 0 when all objects have been
	enumerated."
	<primitive: 139>
	self Error signalPrimitiveFailed.
	^nil! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'Ahe 9/3/2008 19:01'!
notNil "^<True>"
	"Answers true.
	Should not be overridde in subclasses."
	^true! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'ryan 6/8/2011 09:27'!
out
	self printString out.! !

!ImplementationBase methodsFor: 'squeak-inspecting' stamp: 'ryan 6/21/2011 09:05'!
outboundPointersDo: aBlock
"do aBlock for every object I point to, exactly how the garbage collector would. Adapted from PointerFinder >> #followObject:"

	aBlock value: self class.
	1 to: self class instSize do: [:i | aBlock value: (self instVarAt: i)].
	1 to: self basicSize do: [:i | aBlock value: (self basicAt: i)].! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'Ahe 9/2/2008 17:55'!
perform: selector
	"Sends the message with the given selector to the receiver and answers the result.
	Should not be overridden in subclasses."
	<primitive: 83>
	^self perform: selector withArguments: (self Array new)! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'Ahe 9/2/2008 17:55'!
perform: selector with: arg
	"Sends the message with the given selector and given argument to the receiver and answers the result.
	Should not be overridden in subclasses."
	<primitive: 83>
	^self perform: selector withArguments: (self Array braceWith: arg)! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'Ahe 9/2/2008 17:55'!
perform: selector with: arg1 with: arg2
	"Sends the message with the given selector and given arguments to the receiver and answers the result.
	Should not be overridden in subclasses."
	<primitive: 83>
	^self perform: selector withArguments: (self Array braceWith: arg1 with: arg2)! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'Ahe 9/2/2008 17:55'!
perform: selector with: arg1 with: arg2 with: arg3
	"Sends the message with the given selector and given arguments to the receiver and answers the result.
	Should not be overridden in subclasses."
	<primitive: 83>
	^self perform: selector withArguments: (self Array braceWith: arg1 with: arg2 with: arg3)! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'Ahe 9/2/2008 17:55'!
perform: selector withArguments: args
	"Sends the message with the given selector and given arguments to the receiver and answers the result.
	Should not be overridden in subclasses."
	<primitive: 84>
	self Error signalPrimitiveFailed.
	^nil! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'Ahe 9/2/2008 17:04'!
perform: selector withArguments:  arguments inSuperclass: supercls
	<primitive: 100>
	self Error signalPrimitiveFailed.
	^nil! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'Ahe 9/2/2008 17:04'!
pointsTo: obj
	"Squeak crap: pointer finder"
	<primitive: 132>
	self Error signalPrimitiveFailed.! !

!ImplementationBase methodsFor: 'squeak-inspecting' stamp: 'ryan 6/10/2012 09:47'!
printStringLimitedTo: limit
	"Answer a String whose characters are a description of the receiver.
	If you want to print without a character limit, use fullPrintString."
	| limitedString |
	limitedString := String streamContents: [:s | self printOn: s] limitedTo: limit.
	limitedString size < limit ifTrue: [^ limitedString].
	^ limitedString , '...etc...'! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'Ahe 8/27/2008 14:24'!
rehash! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'GB 7/2/2009 13:33'!
respondsTo: selector "<Symbol>"
	"ObsoletionHelper warnObsolete."
	^self class canUnderstand: selector! !

!ImplementationBase methodsFor: 'squeak-inspecting' stamp: 'ryan 6/21/2011 09:06'!
scaledIdentityHash
	"For identityHash values returned by primitive 75, answer
	such values times 2^18.  Otherwise, match the existing
	identityHash implementation"

	^self identityHash * 262144 "bitShift: 18"! !

!ImplementationBase methodsFor: 'squeak-inspecting' stamp: 'ryan 6/10/2012 09:42'!
shouldBePrintedAsLiteral
	^self isLiteral! !

!ImplementationBase methodsFor: 'squeak-inspecting' stamp: 'ryan 6/10/2012 09:42'!
shouldBePrintedAsLiteralVisiting: aSet
	^self isLiteral! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'Unknown 8/15/2012 10:22'!
species
	^self class! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'Ahe 8/27/2008 14:24'!
tryPrimitive: prim "<SmallInteger>" withArgs: args "<Array>"
	"Used by Squeak debugger"
	<primitive: 118>
	^ ContextPart primitiveFailToken! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'ryan 6/7/2011 21:44'!
warnObsolete
	| obsoleted sender interestingCxt msg |
	obsoleted := thisContext sender home.
	sender := obsoleted sender.
	interestingCxt := self interestingContextOf: sender.
	msg := interestingCxt name, ' sends obsolete message ', obsoleted name.
	"Smalltalk at: #HopscotchGlobalState ifPresent: [ :hgs |
		hgs scheduleUIAction: [Transcript cr; show: msg].
		^self]."
	Transcript cr; show: msg! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'Ahe 9/2/2008 17:06'!
withArgs: args "<Array>" executeMethod: m "<CompiledMethod>"
	<primitive: 188>
	self Error signalPrimitiveFailed.
	^nil! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'GB 7/8/2009 04:41'!
~ aSymbol 
	"An extended left section operator. Answer a one-argument block which sends the argument selector to the receiver with the block argument as the argument. If the selector is unary, the block is still one-argument but the argument is ignored."
	"Because Squeak suc^H^H^H blocks are not reentrant we have to be careful and copy them with fixTemps."

	| numArgs |
	numArgs := aSymbol numArgs.
	numArgs = 1 ifTrue:
		[^[:a | self perform: aSymbol with: a] copy fixTemps].
	numArgs = 0 ifTrue:
		[^[:ignored | self perform: aSymbol] copy fixTemps].
	self error: 'zero- or one-argument selector expected'
! !

!ImplementationBase methodsFor: 'as yet unclassified' stamp: 'GB 7/8/2009 04:40'!
~~ anObject
	"Answer whether the receiver and the argument are not the same object 
	(do not have the same object pointer)."

	self == anObject
		ifTrue: [^ false]
		ifFalse: [^ true]! !

ImplementationBase subclass: #NewspeakObject
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NewspeakCore'!

!NewspeakObject commentStamp: '<historical>' prior: 0!
Class NewspeakObject is the root of the object hierarchy in the Newspeak programming language, and provides behavior common to all objects.

For each protocol P in the system, this class provides a method "isP" which answers true if and only if the receiver is an instance of a class which implements the protocol P.

Discussion:

Unlike other systems inspired by Smalltalk, the number of methods in this class is deliberately kept to a minimum. Behavior should only be provided by this class if it should be considered part of the core set of features available to *all* Newspeak programmers.

The security model of the Newspeak programming language is based on capabilities and it is essential that capabilities cannot leak through behavior provided by this class. For example, this class provides no mechanism for accessing instance variables of its instances. Instead such capabilities are provided to the debugger and other tools through mirror based reflection modules.

   Copyright 2008 Cadence Design Systems, Inc.
   
   Licensed under the Apache License, Version 2.0 (the ''License''); you may not use this file except in compliance with the License.  You may obtain a copy of the License at  http://www.apache.org/licenses/LICENSE-2.0!

NewspeakObject subclass: #Assert
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NewspeakCore'!

!Assert class methodsFor: 'as yet unclassified' stamp: 'ryan 6/3/2011 09:41'!
new
	^self basicNew! !

!Assert methodsFor: 'assertions' stamp: 'ryan 6/3/2011 09:41'!
false: condition "<Block | Boolean>"
	"Raises an error if and only if condition is true or evaluates to true."
	(condition isBlock ifTrue: [condition value] ifFalse: [condition]) ifTrue: [
		self signalAssertionError: 'Assertion failed: ', condition printString]! !

!Assert methodsFor: 'signaling' stamp: 'Ahe 8/28/2008 16:18'!
signalAssertionError: message <String>
	^self Error signal: message! !

!Assert methodsFor: 'assertions' stamp: 'ryan 6/3/2011 09:41'!
true: condition "<Block | Boolean>"
	"Raises an error if and only if condition is false or evaluates to false."
	(condition isBlock ifTrue: [condition value] ifFalse: [condition]) ifFalse: [
		self signalAssertionError: 'Assertion failed: ', condition printString]! !

NewspeakObject subclass: #Deployment
	instanceVariableNames: ''
	classVariableNames: 'TheApp'
	poolDictionaries: ''
	category: 'NewspeakCore'!

!Deployment commentStamp: '<historical>' prior: 0!
Holds utility methods for creating a deployment image and loading an application from command line arguments.

Copyright (c) 2010  Ryan Macnak. 

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the ''Software''), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED ''AS IS'', WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.!

!Deployment class methodsFor: 'as-yet-unclassified' stamp: 'Unknown 12/24/2012 00:20'!
appFrom: fn
	| stm deser nof app |	
	
	(fn endsWith: '.gz') ifTrue: [
		stm:= ReadStream on: (GZipReadStream on: (FileStream readOnlyFileNamed: fn)) contents.
	] ifFalse: [
		stm:= (FileStream oldFileNamed: fn) binary.
	].
	
	nof:= NewspeakObjectFormat 	
		usingPlatform: BlackMarket platform
		vmMirror: VMMirror new.
	deser:= nof deserializerFrom: stm.
	app:= deser nextValue.
	deser localHistory do: [:each | [each rehash] ifError: []].
	^app! !

!Deployment class methodsFor: 'as yet unclassified' stamp: 'ryan 7/22/2011 09:57'!
appToExe: app

	| n prevDir vmDir src dest |
	
	self assert: self canMakeExe.
	
	prevDir:= FileDirectory default fullName.
	vmDir:= (FileDirectory on: SmalltalkImage current vmPath) fullName.
	n:= app class name.

	[
		FileDirectory setDefaultDirectory: vmDir.
		self appToNof: app. "1. serialize an object that responds to main:args: in a nof file"
	] ensure: [ 
		FileDirectory setDefaultDirectory: prevDir.
	].

	"2. deseralize said object in a prestine deployment image and arrange that its main:args: will be called on next start up"
	self run: 'nsvm deployment.image ',n,'.nof' in: vmDir.
	"3. put the image into the vm as a resource"
	self run: 'ResHacker -addoverwrite nsvm.exe,',n,'.exe,',n,'.image,123,123,' in: vmDir.
	"4. compress the exe (not strictly necessary)"
	self run: 'upx ',n,'.exe' in: vmDir.

	"delete intermediate files n.nof, n.nof.gz., n.image, n.changes"
	(FileDirectory on: SmalltalkImage current vmPath)
		deleteFileNamed: n,'.nof';
		deleteFileNamed: n,'.nof.gz';
		deleteFileNamed: n,'.image';
		deleteFileNamed: n,'.changes'.

	"move n.exe to current directory (if different than vm directory)"
	src:= (FileDirectory on: SmalltalkImage current vmPath) / (n,'.exe').
	dest:= FileDirectory default / (n,'.exe').
	src rename: src fullName toBe: dest fullName.! !

!Deployment class methodsFor: 'as yet unclassified' stamp: 'Unknown 12/24/2012 00:19'!
appToNof: app

	|stm nof|
	"Serialize app to a NOF file"
	stm:= FileStream forceNewFileNamed: app class name , '.nof'.
	stm binary.
	nof:= NewspeakObjectFormat 	
		usingPlatform: BlackMarket platform
		vmMirror: VMMirror new.
	(nof serializerOn: stm) nextPut: app.
	stm flush; close.
	
	"Also produce a compressed version"
	"stm:= (FileStream oldFileNamed: app class name , '.nof') binary.
	(GZipWriteStream on: 
		(FileStream forceNewFileNamed: app class name , '.nof.gz') binary)
	 		nextPutAll: stm contents;
			flush; close.
	stm close."

	! !

!Deployment class methodsFor: 'as yet unclassified' stamp: 'ryan 2/12/2010 18:59'!
canMakeExe

	^(OSProcess isWindows) and: [
	((FileDirectory on: SmalltalkImage current vmPath) fileExists: 'nsvm.exe') and: [
	((FileDirectory on: SmalltalkImage current vmPath) fileExists: 'deployment.image') and: [
	((FileDirectory on: SmalltalkImage current vmPath) fileExists: 'deployment.changes') and: [
	((FileDirectory on: SmalltalkImage current vmPath) fileExists: 'ResHacker.exe') and: [
	((FileDirectory on: SmalltalkImage current vmPath) fileExists: 'upx.exe') ]]]]]! !

!Deployment class methodsFor: 'as yet unclassified' stamp: 'ryan 6/10/2012 09:40'!
makeDeploymentImage

	WorldState addDeferredUIMessage: [

	|oldWindows|

	"don't complain when we get rid of all windows"
	NativeSession soleInstance watchdogProcess ifNotNil: [:it | it terminate].
	
	"clearTheDesktop"
	oldWindows:= OrderedCollection withAll: BlackMarket platform brazil theDesktop windows.
	oldWindows do: [:ea | ea close].
	oldWindows:= OrderedCollection withAll: BlackMarket platform brazil theDesktop windows.
	oldWindows do: [:ea | BlackMarket platform brazil theDesktop remove: ea].
	oldWindows:= nil.
	ActiveWorld removeAllMorphs.

	NsToolSet releaseMemory: true.

	Smalltalk removeFromStartUpList: AutoStart."Tries to do stuff with args"
	Smalltalk addToStartUpList: self.

	Preferences setPreference: #inPackagedImage toValue: true.
	Preferences 
		addBooleanPreference: #allowDebugger 
		category: 'NsSystem' 
		default: true 
		balloonHelp: 'Help?'.
	Preferences setPreference: #allowDebugger toValue: false.

	self saveAsAndQuit: 'deployment'

	]! !

!Deployment class methodsFor: 'as yet unclassified' stamp: 'Unknown 3/5/2013 21:56'!
makeDeploymentImageFor: app

	WorldState addDeferredUIMessage: [

	|oldWindows|

	"don't complain when we get rid of all windows"
	NativeSession soleInstance watchdogProcess ifNotNil: [:it | it terminate].
	
	"clearTheDesktop"
		oldWindows:= OrderedCollection withAll: BlackMarket platform brazil theDesktop windows.
	oldWindows do: [:ea |  ea close].
	oldWindows:= OrderedCollection withAll: BlackMarket platform brazil theDesktop windows.
	oldWindows do: [:ea | BlackMarket platform brazil theDesktop remove: ea].
	oldWindows:= nil.
	ActiveWorld removeAllMorphs.

	NsToolSet releaseMemory: true.

	ProjectLauncher deactivate.
	Smalltalk removeFromStartUpList: AutoStart."Tries to do stuff with args"
	Smalltalk addToStartUpList: self.

	Preferences setPreference: #inPackagedImage toValue: true.
	Preferences 
		addBooleanPreference: #allowDebugger 
		category: 'NsSystem' 
		default: true 
		balloonHelp: 'Help?'.
	Preferences setPreference: #allowDebugger toValue: false.

	TheApp := app.
	self saveAsAndQuit: app class name

	]! !

!Deployment class methodsFor: 'as yet unclassified' stamp: 'Unknown 12/24/2012 00:24'!
nofDeepCopy: obj

	| bytes nof copy deser |
	
	nof:= NewspeakObjectFormat 	
		usingPlatform: BlackMarket platform
		vmMirror: VMMirror new.
	bytes:= ByteArray streamContents:
		[:stm | (nof serializerOn: stm) nextPut: obj].
	deser:= nof deserializerFrom: bytes readStream.
	copy:= deser nextValue.
	deser localHistory do: [:each | [each rehash] ifError: []].
	^copy
! !

!Deployment class methodsFor: 'as yet unclassified' stamp: 'ryan 2/20/2010 15:03'!
reset
	TheApp:= nil.
	Smalltalk removeFromStartUpList: self.! !

!Deployment class methodsFor: 'as yet unclassified' stamp: 'ryan 2/20/2010 15:04'!
run: cmd in: dir

	"execute cmd with dir as its current directory, and wait for it to finish"

	| si pi cmdStr dirStr |
	si:= Alien newC: 18*4.
	pi:= Alien newC: 4*4.
	cmdStr:= Alien newCString: cmd.
	dirStr:= Alien newCString: dir.
	[

	"si.cb = sizeof(si)"
	si unsignedLongAt: 1 put: 18*4.
	
	((Alien lookup: 'CreateProcessA' inLibrary: 'Kernel32')
		primFFICallResult: (Alien new:4)
		with: 0 "app name"
		with: cmdStr pointer "command line"
		with: 0 "optional"
		with: 0 "optional"
		with: 0 "bool inhertithandles: FALSE"
		with: 16r08000000 "creationflags: NO WINDOW"
		with: 0 "environment"
		with: dirStr pointer "current directory"
		with: si pointer "startup info"
		with: pi pointer "process info"
	) asUnsignedLong = 0 ifFalse: ['error'].

	(Alien lookup: 'WaitForSingleObject' inLibrary: 'Kernel32')
		primFFICallResult: nil
		with: (pi unsignedLongAt: 1) "pi.hProcess"
		with: 16rFFFFFFF.

	(Alien lookup: 'CloseHandle' inLibrary: 'Kernel32')
		primFFICallResult: nil
		with: (pi unsignedLongAt: 1). "pi.hProcess"
	
	(Alien lookup: 'CloseHandle' inLibrary: 'Kernel32')
		primFFICallResult: nil
		with: (pi unsignedLongAt: 5). "pi.hThread"	

	] ensure: [
		si free. pi free. cmdStr free. dirStr free.
	].! !

!Deployment class methodsFor: 'as yet unclassified' stamp: 'ryan 1/4/2010 16:45'!
saveAsAndQuit: name
	SmalltalkImage current saveChangesInFileNamed: name, '.changes'.
	SmalltalkImage current saveImageInFileNamed: name, '.image'.
	SmalltalkImage current closeSourceFiles.
	SmalltalkImage current snapshot: true andQuit: true.! !

!Deployment class methodsFor: 'as yet unclassified' stamp: 'Unknown 9/2/2012 14:49'!
startUp: resuming
	resuming ifFalse: [^self].
	
	WorldState addDeferredUIMessage: [

		|index args attr|
		args := OrderedCollection new.
		index:= 2.
		[attr:= SmalltalkImage current getSystemAttribute: index.
 		attr isNil] whileFalse:
			[args add: attr.
			 index:= index + 1].
		self startUpWithArgs: args.
	].! !

!Deployment class methodsFor: 'as yet unclassified' stamp: 'ryan 2/22/2012 12:12'!
startUpWithArgs: args

	TheApp ifNil: [
		"I am a deployment image without a loaded application.  I expect to be give a single argument naming a NOF file to load."
		args size = 1 ifFalse: [NsToolSet openClassBrowser. ^self].
		TheApp := self appFrom: args first.
		self saveAsAndQuit: TheApp class name.
	] ifNotNil: [
		"I am a deployment image already loaded with an application: start it"
		Smalltalk removeFromStartUpList: self.
		TheApp main: BlackMarket platform args: args	
	].
! !

!NewspeakObject class methodsFor: 'as yet unclassified' stamp: 'ryan 2/17/2011 17:10'!
initialize
	"Remove so it will trigger the DNU and be directed to the proper initializer"
	self methodDictionary removeKey: #'initializer`new' ifAbsent: [].! !

!NewspeakObject class methodsFor: 'as yet unclassified' stamp: 'ryan 12/5/2010 12:05'!
new
	"Should be able to say NewspeakObject new"
	^self basicNew! !

!NewspeakObject methodsFor: 'comparing' stamp: 'Ahe 8/27/2008 15:02'!
= other "^<Boolean>"
	"Answers true if the receiver is considered equal to 'other'.
	May be overridden by subclasses, and should be overridden if the method 'hash' is overridden.
	This method must implement an equivalence relation, that is, the following must hold:
	* x = x evaluates to true
	* if x = y evaluates to true then y = x must evaluate to true
	* if x = y evaluates to true and y = z evaluates to true then x = z must evaluate to true.
	Furthermore, this method must be consistent with the implementation of hash, that is:
	* if x = y evaluates to true then x hash = y hash must evaluate to true."
	^self == other! !

!NewspeakObject methodsFor: 'comparing' stamp: 'Ahe 9/2/2008 16:49'!
== other "^<Boolean>"
	"Answers true if the receiver is the same object as 'other'.
	Should not be overridden by subclasses."
	<primitive: 110>
	self Error signalPrimitiveFailed.
	^false! !

!NewspeakObject methodsFor: 'newspeak requirement' stamp: 'Ahe 8/27/2008 15:10'!
Array "^<Array class>"
	"Answers the class Array or a compatible object.
	May be overridden in subclasses."
	^Array! !

!NewspeakObject methodsFor: 'newspeak requirement' stamp: 'ryan 12/26/2009 21:03'!
ByteArray "^<ByteArray class>"
	"Answers the class ByteArray or a compatible object.
	May be overridden in subclasses."
	^ByteArray! !

!NewspeakObject methodsFor: 'newspeak requirement' stamp: 'Ahe 8/27/2008 15:10'!
Character "^<Character class>"
	"Answers the class Character or a compatible object.
	May be overridden in subclasses."
	^Character! !

!NewspeakObject methodsFor: 'newspeak requirement' stamp: 'Ahe 9/2/2008 16:47'!
Error "^<Error class>"
	"Answers the class Error or a compatible object.
	May be overridden in subclasses."
	^NewspeakError! !

!NewspeakObject methodsFor: 'newspeak requirement' stamp: 'Ahe 8/27/2008 15:10'!
Float "^<Float class>"
	"Answers the class Float or a compatible object.
	May be overridden in subclasses."
	^Float! !

!NewspeakObject methodsFor: 'newspeak requirement' stamp: 'Ahe 8/27/2008 15:10'!
Integer "^<Integer class>"
	"Answers the class Integer or a compatible object.
	May be overridden in subclasses."
	^Integer! !

!NewspeakObject methodsFor: 'newspeak requirement' stamp: 'Ahe 8/27/2008 15:10'!
Object "^<Object class>"
	"Answers the class Object or a compatible object.
	May be overridden in subclasses."
	^NewspeakObject! !

!NewspeakObject methodsFor: 'as yet unclassified' stamp: 'ryan 2/22/2012 12:13'!
Pattern
	| patternMatching |
	patternMatching := Smalltalk 
		at: #CachedPatternMatching
		ifAbsentPut: [NSPatternMatching usingPlatform: BlackMarket platform].
	
	^patternMatching PatternClass
	! !

!NewspeakObject methodsFor: 'newspeak requirement' stamp: 'Ahe 8/27/2008 15:10'!
String "^<String class>"
	"Answers the class String or a compatible object.
	May be overridden in subclasses."
	^String! !

!NewspeakObject methodsFor: 'newspeak requirement' stamp: 'Ahe 8/27/2008 15:10'!
Symbol "^<Symbol class>"
	"Answers the class Symbol or a compatible object.
	May be overridden in subclasses."
	^Symbol! !

!NewspeakObject methodsFor: 'protected' stamp: 'Ahe 9/2/2008 17:54'!
assert: condition "<[^Boolean]>" message: message
	"Raises an error with the given message if and only if condition evaluates to false."
	condition isBlock ifFalse: [self Error signal: 'Block required'].
	condition value ifFalse: [self Error signalAssertionFailure: message]! !

!NewspeakObject methodsFor: 'object synchronization' stamp: 'Ahe 8/27/2008 15:12'!
canonicalRepresentation "^<self | CanonicalRepresentation>"
	"Answers the canonical representation of the receiver, that is, either the receiver itself or an instance of CanonicalRepresentation."
	^self! !

!NewspeakObject methodsFor: 'as yet unclassified' stamp: 'ryan 2/22/2012 12:11'!
case: pattern otherwise: fail 
	| patternMatching MatchFailure tempResult |

	patternMatching := Smalltalk 
		at: #CachedPatternMatching
		ifAbsentPut: [NSPatternMatching usingPlatform: BlackMarket platform].

	MatchFailure:= patternMatching Binding new.
	tempResult:= pattern doesMatch: self else: [ MatchFailure ].

	^ tempResult == MatchFailure
		ifTrue: [ pattern matchFailedFor: self escape: fail ]
		ifFalse: [ tempResult boundValue ].

! !

!NewspeakObject methodsFor: 'accessing' stamp: 'Ahe 9/2/2008 16:48'!
class
	"Answers the class of the receiver.
	Should not be overriden by subclasses."
	<primitive: 111>
	self Error signalPrimitiveFailed.
	^nil! !

!NewspeakObject methodsFor: 'protected' stamp: 'Ahe 9/2/2008 16:53'!
clone
	<primitive: 148>
	self Error signalPrimitiveFailed! !

!NewspeakObject methodsFor: 'error handling' stamp: 'eem 10/3/2011 08:41'!
doesNotUnderstand: msg "<Message>"
	"Sent by the underlying system if an attempt was made to send a message to the receiver that it does not understand, that is, implement.
	May be overriden in subclasses."
	| sel exception resumptionValue mprop |
	sel := msg selector.
	(sel isUnary
		and: [sel size > 2
		and: [(sel at: 1) = $i
		and: [(sel at: 2) = $s
		and: [(sel at: 3) isUppercase]]]]) ifTrue: [ | m |
		"sel looks like 'isP' where P is a protocol.
		Add default implementation of sel returning false."
		"The following construct the method: 'isP ^false'"
		m := CompiledMethod newMethod: 4 header: 1282.
			"objectAt: 1 is the header"
		m objectAt: 2 put: ((Smalltalk includesKey: #AdditionalMethodState)
			ifTrue: [sel]
			ifFalse: [MethodProperties new selector: sel]).
		m objectAt: 3 put: (Smalltalk associationAt: #NewspeakObject).
		"The four bytes of the method are zero which means no source."
		NewspeakObject addSelectorSilently: msg selector withMethod: m.
		^msg sentTo: self].
	
	(sel startsWith: 'initializer`') ifTrue: [ 
		"Coerce NS3 name-managled instance initializer to un-managled NS2 couterpart so NS3 can subclass NS2."
		msg setSelector: (sel allButFirst: 12) asSymbol.
		^msg sentTo: self.
	].
	
	exception := (NewspeakError messageNotUnderstoood: msg receiver: self).
	resumptionValue := exception signal.
	
	^ exception reachedDefaultHandler
		ifTrue: [ msg sentTo: self ]
		ifFalse: [ resumptionValue ].! !

!NewspeakObject methodsFor: 'comparing' stamp: 'Ahe 8/27/2008 14:58'!
hash "^<Integer>"
	"Answers the hash code for the receiver.
	May be overridden by subclasses, and should be overridden if the method '=' is overridden.
	The hash code must obey the requirements set forth in the method '=', that is, the following must hold:
	* if x = y evaluates to true then x hash = y hash must evaluate to true."
	^self identityHash! !

!NewspeakObject methodsFor: 'comparing' stamp: 'Ahe 9/2/2008 16:49'!
identityHash "^<Integer>"
	"Answers the identity hash code of the reciever.
	Should not be overriden by subclasses."
	<primitive: 75>
	self Error signalPrimitiveFailed.
	^0! !

!NewspeakObject methodsFor: 'testing' stamp: 'Ahe 8/27/2008 15:20'!
isImmutable "^<Boolean>"
	"Answer true if and only if the receiver is immutable.
	Should not be overridden in subclasses."
	<primitive: 163>
	^false! !

!NewspeakObject methodsFor: 'printing' stamp: 'eem 10/3/2011 08:45'!
longPrintOn: aStream
	"Append to the argument, aStream, the names and values of all 
	of the receiver's instance variables."

	self flag: 'reimplement with mirrors?'.
	self class allInstVarNames doWithIndex:
		[:title :index |
		aStream nextPutAll: title;
		 nextPut: $:;
		 space;
		 tab;
		 print: (self instVarAt: index);
		 cr]! !

!NewspeakObject methodsFor: 'printing' stamp: 'eem 10/3/2011 08:46'!
longPrintOn: aStream limitedTo: sizeLimit indent: indent
	"Append to the argument, aStream, the names and values of all of the receiver's instance variables.  Limit is the length limit for each inst var."

	self flag: 'reimplement with mirrors?'.
	self class allInstVarNames doWithIndex:
		[:title :index |
		indent timesRepeat: [aStream tab].
		aStream nextPutAll: title;
		 nextPut: $:;
		 space;
		 tab;
		 nextPutAll: 
			((self instVarAt: index) printStringLimitedTo: (sizeLimit -3 -title size max: 1));
		 cr]! !

!NewspeakObject methodsFor: 'newspeak requirement' stamp: 'Ahe 8/27/2008 15:10'!
new
	"Does nothing. May be overriden in subclasses."! !

!NewspeakObject methodsFor: 'printing' stamp: 'Ahe 8/27/2008 15:18'!
printOn: s "<WriteStream> ^<self>"
	"Prints a representation of the receiver on the given stream.
	The printed representation is for debugging purposes only and should not be relied on for any other purpose whatsoever.
	May be overridden in subclasses."
	s
		nextPutAll: self class name;
		nextPut: $#;
		nextPutAll: self identityHash printString! !

!NewspeakObject methodsFor: 'printing' stamp: 'Ahe 8/27/2008 15:19'!
printString "^<String>"
	"Returns a printed representation of the receiver.
	The printed representation is for debugging purposes only and should not be relied on for any other purpose whatsoever.
	May be overridden in subclasses."
	^self String streamContents: [ :s | self printOn: s]! !

!NewspeakObject methodsFor: 'printing' stamp: 'tk 5/7/1999 16:20'!
printStringLimitedTo: limit
	"Answer a String whose characters are a description of the receiver.
	If you want to print without a character limit, use fullPrintString."
	| limitedString |
	limitedString := String streamContents: [:s | self printOn: s] limitedTo: limit.
	limitedString size < limit ifTrue: [^ limitedString].
	^ limitedString , '...etc...'! !

!NewspeakObject methodsFor: 'error handling' stamp: 'GB 10/24/2009 18:35'!
subclassResponsibility
	self Error signal: 'My subclass should have overridden ', thisContext sender selector printString! !

!NewspeakObject methodsFor: 'accessing' stamp: 'Ahe 8/28/2008 14:39'!
yourself! !

NewspeakObject subclass: #ObsoletionHelper
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NewspeakCore'!

!ObsoletionHelper class methodsFor: 'as yet unclassified' stamp: 'Ahe 8/28/2008 12:35'!
interestingContextOf: context "<ContextPart>"
	| interestingContext |
	interestingContext := context.
	[interestingContext selector includes: Language syntheticNameSeparatorCharacter]
		whileTrue: [interestingContext := interestingContext sender].
	^interestingContext! !

!ObsoletionHelper class methodsFor: 'as yet unclassified' stamp: 'GB 9/30/2009 15:45'!
warnObsolete
	| obsoleted sender interestingCxt msg |
	obsoleted := thisContext sender home.
	sender := obsoleted sender.
	interestingCxt := self interestingContextOf: sender.
	msg := interestingCxt name, ' sends obsolete message ', obsoleted name.
	"Smalltalk at: #HopscotchGlobalState ifPresent: [ :hgs |
		hgs scheduleUIAction: [Transcript cr; show: msg].
		^self]."
	Transcript cr; show: msg! !

NewspeakObject subclass: #VMMirror
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NewspeakCore'!

!VMMirror commentStamp: '<historical>' prior: 0!
Newspeak has no syntax for primitives, instead such functionality is achieved by message sends to the vm mirror.  The API of this class is immutature and may be subject to major change.

N.B.: The trick used here to call primitives will leave the stack unbalanced.  Primitives will generally pop the number of arguments they expect, rather than how many they actually receive.  This is okay as long as a call to a such a method is not part of a compound statement or in a loop.!

!VMMirror methodsFor: 'SmallInteger' stamp: 'ryan 9/7/2010 20:32'!
add: a "SmallInteger" and: b "SmallInteger" ifFail: blk
	^self ifFail: blk add: a and: b! !

!VMMirror methodsFor: 'FFI' stamp: 'ryan 10/24/2010 15:52'!
alien: alien addressFieldPut: value "<Integer> ^<Integer>" ifFail: blk 
	^self ifFail: blk alien: alien addressFieldPut: value! !

!VMMirror methodsFor: 'FFI' stamp: 'ryan 10/25/2010 17:38'!
alien: alien copyInto: aByteObject "<Alien | indexableByteSubclass>" from: start "<Integer>" to: stop "<Integer>" in: replacement "<Alien | indexableByteSubclass>" startingAt: repStart "<Integer>" "^<self>" ifFail: blk
	^self ifFail: blk alien: alien copyInto: aByteObject from: start to: stop in: replacement startingAt: repStart! !

!VMMirror methodsFor: 'FFI' stamp: 'ryan 10/24/2010 17:45'!
alien: alien doubleAt: index "<Integer> ^<Float>" ifFail: blk 
	^self ifFail: blk alien: alien doubleAt: index! !

!VMMirror methodsFor: 'FFI' stamp: 'ryan 10/24/2010 17:43'!
alien: alien doubleAt: index "<Integer>" put: value "<Float | Integer> ^<Float | Integer>" ifFail: blk 
	^self ifFail: blk alien: alien doubleAt: index put: value! !

!VMMirror methodsFor: 'FFI' stamp: 'ryan 10/24/2010 17:47'!
alien: alien floatAt: index "<Integer> ^<Float>" ifFail: blk
	^self ifFail: blk alien: alien floatAt: index! !

!VMMirror methodsFor: 'FFI' stamp: 'ryan 10/24/2010 17:48'!
alien: alien floatAt: index "<Integer>" put: value "<Float | Integer> ^<Float | Integer>" ifFail: blk 
	^self ifFail: blk alien: alien floatAt: index put: value! !

!VMMirror methodsFor: 'FFI' stamp: 'ryan 10/25/2010 17:37'!
alien: alien  replaceFrom: start "<Integer>" to: stop "<Integer>" with: replacement "<Alien | indexableByteSubclass>" startingAt: repStart "<Integer>" "^<self>" ifFail: blk 
	^self ifFail: blk alien: alien replaceFrom: start to: stop with: replacement startingAt: repStart! !

!VMMirror methodsFor: 'FFI' stamp: 'ryan 10/24/2010 16:02'!
alien: alien signedByteAt: index "<Integer> ^<Integer>" ifFail: blk 
	^self ifFail: blk alien: alien signedByteAt: index! !

!VMMirror methodsFor: 'FFI' stamp: 'ryan 10/24/2010 16:04'!
alien: alien signedByteAt: index "<Integer>" put: value "<Integer> ^<Integer>" ifFail: blk
	^self ifFail: blk alien: alien signedByteAt: index put: value! !

!VMMirror methodsFor: 'FFI' stamp: 'ryan 10/25/2010 17:21'!
alien: alien signedLongAt: index "<Integer> ^<Integer>" ifFail: blk 
	^self ifFail: blk alien: alien signedLongAt: index! !

!VMMirror methodsFor: 'FFI' stamp: 'ryan 10/25/2010 17:22'!
alien: alien signedLongAt: index "<Integer>" put: value "<Integer> ^<Integer>" ifFail: blk 
	^self ifFail: blk alien: alien signedLongAt: index put: value! !

!VMMirror methodsFor: 'FFI' stamp: 'ryan 10/25/2010 17:23'!
alien: alien signedShortAt: index "<Integer> ^<Integer>" ifFail: blk
	^self ifFail: blk alien: alien signedShortAt: index! !

!VMMirror methodsFor: 'FFI' stamp: 'ryan 10/25/2010 17:25'!
alien: alien signedShortAt: index "<Integer>" put: value "<Integer> ^<Integer>" ifFail: blk
	^self ifFail: blk alien: alien signedShortAt: index put: value! !

!VMMirror methodsFor: 'FFI' stamp: 'ryan 10/24/2010 15:28'!
alien: alien sizeFieldPut: size "<Integer> ^<Integer>" ifFail: blk 
	^self ifFail: blk alien: alien sizeFieldPut: size! !

!VMMirror methodsFor: 'FFI' stamp: 'ryan 10/25/2010 17:34'!
alien: alien strlenStartingAt: index "<Integer> ^<Integer>" ifFail: blk 
	^self ifFail: blk alien: alien strlenStartingAt: index! !

!VMMirror methodsFor: 'FFI' stamp: 'ryan 10/25/2010 17:35'!
alien: alien strlenThroughPointerAt: index "<Integer> ^<Integer>" ifFail: blk 
	^self ifFail: blk alien: alien strlenThroughPointerAt: index! !

!VMMirror methodsFor: 'FFI' stamp: 'ryan 10/25/2010 17:33'!
alien: alien unsignedByteAt: index "<Integer> ^<Integer>" ifFail: blk 
	^self ifFail: blk alien: alien unsignedByteAt: index ! !

!VMMirror methodsFor: 'FFI' stamp: 'ryan 10/25/2010 17:31'!
alien: alien unsignedByteAt: index "<Integer>" put: value "<Integer> ^<Integer>" ifFail: blk 
	^self ifFail: blk alien: alien unsignedByteAt: index put: value! !

!VMMirror methodsFor: 'FFI' stamp: 'ryan 10/25/2010 17:29'!
alien: alien unsignedLongAt: index "<Integer> ^<Integer>" ifFail: blk 
	^self ifFail: blk alien: alien unsignedLongAt: index ! !

!VMMirror methodsFor: 'FFI' stamp: 'ryan 10/25/2010 17:30'!
alien: alien unsignedLongAt: index "<Integer>" put: value "<Integer> ^<Integer>" ifFail: blk 
	^self ifFail: blk alien: alien unsignedLongAt: index put: value! !

!VMMirror methodsFor: 'FFI' stamp: 'ryan 10/25/2010 17:26'!
alien: alien unsignedShortAt: index "<Integer> ^<Integer>" ifFail: blk 
	^self ifFail: blk alien: alien unsignedShortAt: index! !

!VMMirror methodsFor: 'FFI' stamp: 'ryan 10/25/2010 17:28'!
alien: alien unsignedShortAt: index "<Integer>" put: value "<Integer> ^<Integer>" ifFail: blk 
	^self ifFail: blk alien: alien unsignedShortAt: index put: value! !

!VMMirror methodsFor: 'FFI' stamp: 'ryan 10/24/2010 15:51'!
alienAddressField: alien ifFail: blk "^<Integer>"
	^self ifFail: blk alienAddressField: alien! !

!VMMirror methodsFor: 'FFI' stamp: 'ryan 10/24/2010 15:33'!
alienSizeField: alien ifFail: blk "^<Integer>"
	^self ifFail: blk alienSizeField: alien! !

!VMMirror methodsFor: 'Float' stamp: 'ryan 9/7/2010 20:33'!
asFloat: a ifFail: blk
	^self ifFail: blk asFloat: a! !

!VMMirror methodsFor: 'SmallInteger' stamp: 'ryan 9/7/2010 20:35'!
bitAnd: a "SmallInteger" and: b "SmallInteger" ifFail: blk
	^self ifFail: blk bitAnd: a and: b! !

!VMMirror methodsFor: 'SmallInteger' stamp: 'ryan 9/7/2010 20:35'!
bitOr: a "SmallInteger" and: b "SmallInteger" ifFail: blk
	^self ifFail: blk bitOr: a and: b! !

!VMMirror methodsFor: 'SmallInteger' stamp: 'ryan 9/7/2010 20:36'!
bitShift: a "SmallInteger" by: b "SmallInteger" ifFail: blk
	^self ifFail: blk bitShift: a and: b! !

!VMMirror methodsFor: 'SmallInteger' stamp: 'ryan 9/7/2010 20:36'!
bitXor: a "SmallInteger" and: b "SmallInteger" ifFail: blk
	^self ifFail: blk bitXor: a and: b! !

!VMMirror methodsFor: 'as yet unclassified' stamp: 'ryan 2/22/2012 12:13'!
blackMarket
	^BlackMarket new! !

!VMMirror methodsFor: 'FFI' stamp: 'ryan 10/24/2010 14:12'!
callFunction: function "<Alien>" doubleResult: result "<Alien | nil >" arguments: args "<Array[Alien | Integer]>" ifFail: blk "<[Symbol]> ^<Alien>" 

	^self ifFail: blk callFunction: function doubleResult: result arguments: args! !

!VMMirror methodsFor: 'FFI' stamp: 'ryan 10/24/2010 14:12'!
callFunction: function "<Alien>" floatResult: result "<Alien | nil >" arguments: args "<Array[Alien | Integer]>" ifFail: blk "<[Symbol]> ^<Alien>" 

	^self ifFail: blk callFunction: function floatResult: result arguments: args! !

!VMMirror methodsFor: 'FFI' stamp: 'ryan 10/24/2010 14:09'!
callFunction: function "<Alien>" integralResult: result "<Alien | nil >" arguments: args "<Array[Alien | Integer]>" ifFail: blk "<[Symbol]> ^<Alien>" 

	^self ifFail: blk callFunction: function integralResult: result arguments: args! !

!VMMirror methodsFor: 'FFI' stamp: 'ryan 10/19/2010 14:40'!
calloc: byteSize "<Integer> ^<Integer>" ifFail: blk
	^self ifFail: blk calloc: byteSize! !

!VMMirror methodsFor: 'Object' stamp: 'Unknown 11/12/2012 21:53'!
changeClassOf: someObject to: newClass ifFail: blk 
	^self ifFail: blk haveClass: newClass adoptInstance: someObject! !

!VMMirror methodsFor: 'Object' stamp: 'ryan 1/18/2010 00:00'!
classOf: a
	<primitive: 111>! !

!VMMirror methodsFor: 'Object' stamp: 'ryan 11/11/2011 22:38'!
clone: obj
	<primitive: 148>
	^nil primitiveFailed! !

!VMMirror methodsFor: 'SmallInteger' stamp: 'ryan 9/9/2010 15:41'!
div: a "SmallInteger" and: b "SmallInteger" ifFail: blk
	^self ifFail: blk div: a and: b! !

!VMMirror methodsFor: 'SmallInteger' stamp: 'ryan 9/9/2010 15:42'!
divide: a "SmallInteger" and: b "SmallInteger" ifFail: blk
	^self ifFail: blk divide: a and: b! !

!VMMirror methodsFor: 'Object' stamp: 'ryan 7/21/2011 14:13'!
firstInstanceOf: aBehavior ifNone: blk
	^self ifNone: blk firstInstanceOf: aBehavior! !

!VMMirror methodsFor: 'Float' stamp: 'ryan 9/9/2010 15:31'!
floatAdd: a and: b ifFail: blk
	^self ifFail: blk floatAdd: a and: b! !

!VMMirror methodsFor: 'Float' stamp: 'ryan 9/9/2010 15:32'!
floatDivide: a and: b ifFail: blk
	^self ifFail: blk floatDivide: a and: b! !

!VMMirror methodsFor: 'Float' stamp: 'ryan 9/9/2010 15:32'!
floatExponent: a ifFail: blk
	^self ifFail: blk floatExponent: a! !

!VMMirror methodsFor: 'Float' stamp: 'ryan 9/9/2010 15:33'!
floatFractionPart: a ifFail: blk
	^self ifFail: blk floatFractionPart: a! !

!VMMirror methodsFor: 'Float' stamp: 'ryan 9/9/2010 15:33'!
floatIs: a equalTo: b ifFail: blk
	^self ifFail: blk floatIs: a equalTo: b! !

!VMMirror methodsFor: 'Float' stamp: 'ryan 9/9/2010 15:33'!
floatIs: a greaterOrEqual: b ifFail: blk
	^self ifFail: blk floatIs: a greaterOrEqual: b! !

!VMMirror methodsFor: 'Float' stamp: 'ryan 9/9/2010 15:34'!
floatIs: a greaterThan: b ifFail: blk
	^self ifFail: blk floatIs: a greaterThan: b! !

!VMMirror methodsFor: 'Float' stamp: 'ryan 9/9/2010 15:34'!
floatIs: a lessOrEqual: b ifFail: blk
	^self ifFail: blk floatIs: a lessOrEqual: b! !

!VMMirror methodsFor: 'Float' stamp: 'ryan 9/9/2010 15:34'!
floatIs: a lessThan: b ifFail: blk
	^self ifFail: blk floatIs: a lessThan: b! !

!VMMirror methodsFor: 'Float' stamp: 'ryan 9/9/2010 15:35'!
floatIs: a notEqualTo: b ifFail: blk
	^self ifFail: blk floatIs: a notEqualTo: b! !

!VMMirror methodsFor: 'Float' stamp: 'ryan 9/9/2010 15:36'!
floatMultiply: a and: b ifFail: blk
	^self ifFail: blk floatMultiply: a and: b! !

!VMMirror methodsFor: 'Float' stamp: 'ryan 9/9/2010 15:36'!
floatSubtract: a and: b ifFail: blk
	^self ifFail: blk floatSubtract: a and: b! !

!VMMirror methodsFor: 'Float' stamp: 'ryan 9/9/2010 15:36'!
floatTruncated: a ifFail: blk
	^self ifFail: blk floatTruncated: a! !

!VMMirror methodsFor: 'as yet unclassified' stamp: 'ryan 11/11/2011 23:35'!
flushCache
	"Tell the interpreter to remove the contents of its method lookup cache, if it has 
	one.  Essential.  See Object documentation whatIsAPrimitive."

	<primitive: 89>
	nil primitiveFailed! !

!VMMirror methodsFor: 'FFI' stamp: 'ryan 10/24/2010 15:26'!
free: alien ifFail: blk
	"Free the memory referenced by the receiver, or fail if
	 the receiver is direct or the indirect pointer is invalid.
	 Note that if successful the address and size fields are
	 zero'ed, effectively preventing double-freeing hazards."
	^self ifFail: blk free: alien! !

!VMMirror methodsFor: 'Object' stamp: 'ryan 1/17/2010 23:56'!
fullGC
	<primitive: 130>! !

!VMMirror methodsFor: 'Object' stamp: 'ryan 6/8/2010 20:27'!
identityHashOf: object
	"Equivalent to ProtoObject>>identityHash."
	<primitive: 75>
	^object  "Fails if object is a SmallInteger, whose identity hash should be itself"! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/7/2010 20:32'!
ifFail: blk add: a and: b
	<primitive: 1>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 10/24/2010 15:52'!
ifFail: blk alien: alien addressFieldPut: value "<Integer> ^<Integer>"
	<primitive: 'primAddressFieldPut' error: errorCode module: 'IA32ABI'>
	^blk valueWithPossibleArgument: errorCode! !

!VMMirror methodsFor: 'private' stamp: 'ryan 10/25/2010 17:38'!
ifFail: blk alien: alien copyInto: aByteObject "<Alien | indexableByteSubclass>" from: start "<Integer>" to: stop "<Integer>" in: replacement "<Alien | indexableByteSubclass>" startingAt: repStart "<Integer>" "^<self>"
	<primitive: 'primAlienReplace' error: errorCode module: 'IA32ABI'>
	^blk valueWithPossibleArgument: errorCode! !

!VMMirror methodsFor: 'private' stamp: 'ryan 10/24/2010 17:45'!
ifFail: blk alien: alien doubleAt: index "<Integer> ^<Float>"
	<primitive: 'primDoubleAt' error: errorCode module: 'IA32ABI'>
	^blk valueWithPossibleArgument: errorCode! !

!VMMirror methodsFor: 'private' stamp: 'ryan 10/24/2010 17:42'!
ifFail: blk alien: alien doubleAt: index "<Integer>" put: value "<Float | Integer> ^<Float | Integer>"
	<primitive: 'primDoubleAtPut' error: errorCode module: 'IA32ABI'>
	^blk valueWithPossibleArgument: errorCode! !

!VMMirror methodsFor: 'private' stamp: 'ryan 10/24/2010 17:47'!
ifFail: blk alien: alien floatAt: index "<Integer> ^<Float>"
	<primitive: 'primFloatAt' error: errorCode module: 'IA32ABI'>
	^blk valueWithPossibleArgument: errorCode! !

!VMMirror methodsFor: 'private' stamp: 'ryan 10/24/2010 17:49'!
ifFail: blk alien: alien floatAt: index "<Integer>" put: value "<Float | Integer> ^<Float | Integer>"
	<primitive: 'primFloatAtPut' error: errorCode module: 'IA32ABI'>
	^blk valueWithPossibleArgument: errorCode! !

!VMMirror methodsFor: 'private' stamp: 'ryan 10/25/2010 17:37'!
ifFail: blk alien: alien replaceFrom: start "<Integer>" to: stop "<Integer>" with: replacement "<Alien | indexableByteSubclass>" startingAt: repStart "<Integer>" "^<self>"
	<primitive: 'primAlienReplace' error: errorCode module: 'IA32ABI'>
	^blk valueWithPossibleArgument: errorCode! !

!VMMirror methodsFor: 'private' stamp: 'ryan 10/24/2010 16:02'!
ifFail: blk alien: alien signedByteAt: index "<Integer> ^<Integer>"
	<primitive: 'primSignedByteAt' error: errorCode module: 'IA32ABI'>
	^blk valueWithPossibleArgument: errorCode! !

!VMMirror methodsFor: 'private' stamp: 'ryan 10/24/2010 16:04'!
ifFail: blk alien: alien signedByteAt: index "<Integer>" put: value "<Integer> ^<Integer>"
	<primitive: 'primSignedByteAtPut' error: errorCode module: 'IA32ABI'>
	^blk valueWithPossibleArgument: errorCode! !

!VMMirror methodsFor: 'private' stamp: 'ryan 10/25/2010 17:20'!
ifFail: blk alien: alien signedLongAt: index "<Integer> ^<Integer>"
	<primitive: 'primSignedLongAt' error: errorCode module: 'IA32ABI'>
	^blk valueWithPossibleArgument: errorCode! !

!VMMirror methodsFor: 'private' stamp: 'ryan 10/25/2010 17:22'!
ifFail: blk alien: alien signedLongAt: index "<Integer>" put: value "<Integer> ^<Integer>"
	<primitive: 'primSignedLongAtPut' error: errorCode module: 'IA32ABI'>
	^blk valueWithPossibleArgument: errorCode! !

!VMMirror methodsFor: 'private' stamp: 'ryan 10/25/2010 17:23'!
ifFail: blk alien: alien signedShortAt: index "<Integer> ^<Integer>"
	<primitive: 'primSignedShortAt' error: errorCode module: 'IA32ABI'>
	^blk valueWithPossibleArgument: errorCode! !

!VMMirror methodsFor: 'private' stamp: 'ryan 10/25/2010 17:24'!
ifFail: blk alien: alien signedShortAt: index "<Integer>" put: value "<Integer> ^<Integer>"
	<primitive: 'primSignedShortAtPut' error: errorCode module: 'IA32ABI'>
	^blk valueWithPossibleArgument: errorCode! !

!VMMirror methodsFor: 'private' stamp: 'ryan 10/24/2010 15:28'!
ifFail: blk alien: alien sizeFieldPut: size "<Integer> ^<Integer>"
	<primitive: 'primSizeFieldPut' error: errorCode module: 'IA32ABI'>
	^blk valueWithPossibleArgument: errorCode! !

!VMMirror methodsFor: 'private' stamp: 'ryan 10/25/2010 17:34'!
ifFail: blk alien: alien strlenStartingAt: index "<Integer> ^<Integer>"
	<primitive: 'primStrlenFromStartIndex' error: errorCode module: 'IA32ABI'>
	^blk valueWithPossibleArgument: errorCode! !

!VMMirror methodsFor: 'private' stamp: 'ryan 10/25/2010 17:35'!
ifFail: blk alien: alien strlenThroughPointerAt: index "<Integer> ^<Integer>"
	<primitive: 'primStrlenThroughPointerAtIndex' error: errorCode module: 'IA32ABI'>
	^blk valueWithPossibleArgument: errorCode! !

!VMMirror methodsFor: 'private' stamp: 'ryan 10/25/2010 17:32'!
ifFail: blk alien: alien unsignedByteAt: index "<Integer> ^<Integer>"
	<primitive: 'primUnsignedByteAt' error: errorCode module: 'IA32ABI'>
	^blk valueWithPossibleArgument: errorCode! !

!VMMirror methodsFor: 'private' stamp: 'ryan 10/25/2010 17:31'!
ifFail: blk alien: alien unsignedByteAt: index "<Integer>" put: value "<Integer> ^<Integer>"
	<primitive: 'primUnsignedByteAtPut' error: errorCode module: 'IA32ABI'>
	^blk valueWithPossibleArgument: errorCode! !

!VMMirror methodsFor: 'private' stamp: 'ryan 10/25/2010 17:29'!
ifFail: blk alien: alien unsignedLongAt: index "<Integer> ^<Integer>"
	<primitive: 'primUnsignedLongAt' error: errorCode module: 'IA32ABI'>
	^blk valueWithPossibleArgument: errorCode! !

!VMMirror methodsFor: 'private' stamp: 'ryan 10/25/2010 17:30'!
ifFail: blk alien: alien unsignedLongAt: index "<Integer>" put: value "<Integer> ^<Integer>"
	<primitive: 'primUnsignedLongAtPut' error: errorCode module: 'IA32ABI'>
	^blk valueWithPossibleArgument: errorCode! !

!VMMirror methodsFor: 'private' stamp: 'ryan 10/25/2010 17:26'!
ifFail: blk alien: alien unsignedShortAt: index "<Integer> ^<Integer>"
	<primitive: 'primUnsignedShortAt' error: errorCode module: 'IA32ABI'>
	^blk valueWithPossibleArgument: errorCode! !

!VMMirror methodsFor: 'private' stamp: 'ryan 10/25/2010 17:27'!
ifFail: blk alien: alien unsignedShortAt: index "<Integer>" put: value "<Integer> ^<Integer>"
	<primitive: 'primUnsignedShortAtPut' error: errorCode module: 'IA32ABI'>
	^blk valueWithPossibleArgument: errorCode! !

!VMMirror methodsFor: 'private' stamp: 'ryan 10/24/2010 15:51'!
ifFail: blk alienAddressField: alien "^<Integer>"
	<primitive: 'primAddressField' error: errorCode module: 'IA32ABI'>
	^blk valueWithPossibleArgument: errorCode! !

!VMMirror methodsFor: 'private' stamp: 'ryan 10/24/2010 15:33'!
ifFail: blk alienSizeField: alien "^<Integer>"
	<primitive: 'primSizeField' error: errorCode module: 'IA32ABI'>
	^blk valueWithPossibleArguments: errorCode! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/7/2010 20:33'!
ifFail: blk asFloat: f
	<primitive: 40>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/7/2010 20:32'!
ifFail: blk bitAnd: a and: b
	<primitive: 14>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/7/2010 20:34'!
ifFail: blk bitOr: a and: b
	<primitive: 15>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/7/2010 20:34'!
ifFail: blk bitShift: a and: b
	<primitive: 17>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/7/2010 20:35'!
ifFail: blk bitXor: a and: b
	<primitive: 16>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 10/24/2010 14:10'!
ifFail: blk "<[Symbol]>" callFunction: function "<Alien>" doubleResult: result "<Alien | nil >" arguments: args "<Array[Alien | Integer]> ^<Alien>"

	<primitive: 'primVarArgsCallOutDoubleReturn' error: errorCode module: 'IA32ABI'>
	^blk valueWithPossibleArgument: errorCode! !

!VMMirror methodsFor: 'private' stamp: 'ryan 10/24/2010 14:11'!
ifFail: blk "<[Symbol]>" callFunction: function "<Alien>" floatResult: result "<Alien | nil >" arguments: args "<Array[Alien | Integer]> ^<Alien>"

	<primitive: 'primVarArgsCallOutFloatReturn' error: errorCode module: 'IA32ABI'>
	^blk valueWithPossibleArgument: errorCode! !

!VMMirror methodsFor: 'private' stamp: 'ryan 10/24/2010 14:09'!
ifFail: blk "<[Symbol]>" callFunction: function "<Alien>" integralResult: result "<Alien | nil >" arguments: args "<Array[Alien | Integer]> ^<Alien>"

	<primitive: 'primVarArgsCallOutIntegralReturn' error: errorCode module: 'IA32ABI'>
	^blk valueWithPossibleArgument: errorCode! !

!VMMirror methodsFor: 'private' stamp: 'ryan 10/19/2010 14:40'!
ifFail: blk calloc: byteSize "<Integer> ^<Integer>"
	<primitive: 'primCalloc' error: errorCode module: 'IA32ABI'>
	^blk valueWithPossibleArgument: errorCode! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/7/2010 20:36'!
ifFail: blk div: a and: b
	<primitive: 12>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/7/2010 20:37'!
ifFail: blk divide: a and: b
	<primitive: 10>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/7/2010 20:38'!
ifFail: blk floatAdd: a and: b
	<primitive: 41>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/7/2010 20:38'!
ifFail: blk floatDivide: a and: b
	<primitive: 50>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/7/2010 20:38'!
ifFail: blk floatExponent: a
	<primitive: 53>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/7/2010 20:39'!
ifFail: blk floatFractionPart: a
	<primitive: 52>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/7/2010 20:39'!
ifFail: blk floatIs: a equalTo: b
	<primitive: 47>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/7/2010 20:40'!
ifFail: blk floatIs: a greaterOrEqual: b
	<primitive: 46>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/7/2010 20:40'!
ifFail: blk floatIs: a greaterThan: b
	<primitive: 44>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/7/2010 20:40'!
ifFail: blk floatIs: a lessOrEqual: b
	<primitive: 45>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/7/2010 20:41'!
ifFail: blk floatIs: a lessThan: b
	<primitive: 43>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/7/2010 20:41'!
ifFail: blk floatIs: a notEqualTo: b
	<primitive: 48>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/7/2010 20:41'!
ifFail: blk floatMultiply: a and: b
	<primitive: 49>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/7/2010 20:42'!
ifFail: blk floatSubtract: a and: b
	<primitive: 42>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/7/2010 20:42'!
ifFail: blk floatTruncated: a 
	<primitive: 51>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 10/24/2010 15:26'!
ifFail: blk free: alien
	"Free the memory referenced by the receiver, or fail if
	 the receiver is direct or the indirect pointer is invalid.
	 Note that if successful the address and size fields are
	 zero'ed, effectively preventing double-freeing hazards."
	<primitive: 'primBoxedFree' error: errorCode module: 'IA32ABI'>
	^blk valueWithPossibleArgument: errorCode! !

!VMMirror methodsFor: 'private' stamp: 'Unknown 11/12/2012 21:52'!
ifFail: blk haveClass: newClass adoptInstance: someObject
	<primitive: 160 error: ec>
	^blk valueWithPossibleArgument: ec! !

!VMMirror methodsFor: 'private' stamp: 'ryan 10/25/2010 17:39'!
ifFail: blk inLibrary: alien findSymbol: symbolName "<String> ^<Integer>"
	"The receiver is assumed to be a proxy whose first field contans the handle of a library
	 answered by primLoadLibrary.  Attempt to find symbolName in the library and answer
	 its address, or fail if the receiver is invalid or the symbol cannot be found."
	<primitive: 'primInLibraryFindSymbol' error: errorCode module: 'IA32ABI'>
	^blk valueWithPossibleArgument: errorCode! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/9/2010 15:18'!
ifFail: blk indexedSlotOf: object at: index
	"Equivalent to object basicAt: index"
	<primitive: 60>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/9/2010 15:19'!
ifFail: blk indexedSlotOf: object at: index put: value
	"Equivalent to object basicAt: index put: value"
	<primitive: 61>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/9/2010 15:19'!
ifFail: blk instaniateFixedClass: klass
	"Equivalent to Behavior>>basicNew"
	<primitive: 70>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/9/2010 15:20'!
ifFail: blk instaniateVariableClass: klass withSize: n
	"Equivalent to Behavior>>basicNew: n"
	<primitive: 71>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'eem 9/14/2012 15:42'!
ifFail: blk instantiateFixedClass: klass
	"Equivalent to Behavior>>basicNew"
	<primitive: 70>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'eem 9/14/2012 15:42'!
ifFail: blk instantiateVariableClass: klass withSize: n
	"Equivalent to Behavior>>basicNew: n"
	<primitive: 71>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/9/2010 15:20'!
ifFail: blk is: a equalTo: b
	<primitive: 7>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/9/2010 15:21'!
ifFail: blk is: a greaterOrEqual: b
	<primitive: 6>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/9/2010 15:21'!
ifFail: blk is: a greaterThan: b
	<primitive: 4>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/9/2010 15:22'!
ifFail: blk is: a lessOrEqual: b
	<primitive: 5>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/9/2010 15:23'!
ifFail: blk is: a lessThan: b
	<primitive: 3>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/9/2010 15:23'!
ifFail: blk is: a notEqualTo: b
	<primitive: 8>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 10/19/2010 14:54'!
ifFail: blk loadLibrary: libraryName "<String> ^<Integer>" 
	"Attempt to load the shared library/DLL named libraryName and
	 answer a handle which can be used in primFindSymbol:inLibrary:,
	 or fail if libraryName is invalid or libraryName cannot be found."
	<primitive: 'primLoadLibrary' error: errorCode module: 'IA32ABI'>
	^blk valueWithPossibleArgument: errorCode! !

!VMMirror methodsFor: 'private' stamp: 'ryan 10/19/2010 14:43'!
ifFail: blk malloc: byteSize "<Integer> ^<Integer>"
	<primitive: 'primMalloc' error: errorCode module: 'IA32ABI'>
	^blk valueWithPossibleArgument: errorCode! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/9/2010 15:24'!
ifFail: blk mod: a and: b
	<primitive: 11>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/9/2010 15:24'!
ifFail: blk multiply: a and: b
	<primitive: 9>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/9/2010 15:24'!
ifFail: blk namedSlotOf: object at: index
	"Equivalent to object instVarAt: index"
	<primitive: 73>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/9/2010 15:25'!
ifFail: blk namedSlotOf: object at: index put: value
	"Equivalent to object instVarAt: index put: value"
	<primitive: 74>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 11/10/2010 10:44'!
ifFail: blk object: o with: args executeMethod: m 
	<primitive: 188>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/9/2010 15:25'!
ifFail: blk quo: a and: b
	<primitive: 13>
	^blk value! !

!VMMirror methodsFor: 'private' stamp: 'ryan 9/9/2010 15:26'!
ifFail: blk subtract: a and: b
	<primitive: 2>
	^blk value! !

!VMMirror methodsFor: 'as yet unclassified' stamp: 'ryan 7/21/2011 14:13'!
ifNone: blk firstInstanceOf: aBehavior
	<primitive: 77>
	^blk value! !

!VMMirror methodsFor: 'FFI' stamp: 'ryan 10/25/2010 17:40'!
inLibrary: alien findSymbol: symbolName "<String> ^<Integer>" ifFail: blk
	"The receiver is assumed to be a proxy whose first field contans the handle of a library
	 answered by primLoadLibrary.  Attempt to find symbolName in the library and answer
	 its address, or fail if the receiver is invalid or the symbol cannot be found."
	^self ifFail: blk inLibrary: alien findSymbol: symbolName! !

!VMMirror methodsFor: 'Object' stamp: 'ryan 1/17/2010 23:57'!
incrementalGC
	<primitive: 131>! !

!VMMirror methodsFor: 'Object' stamp: 'ryan 9/9/2010 15:39'!
indexedSlotOf: object at: index ifFail: block
	"Equivalent to object basicAt: index"
	^self ifFail: block indexedSlotOf: object at: index! !

!VMMirror methodsFor: 'Object' stamp: 'ryan 9/9/2010 15:39'!
indexedSlotOf: object at: index put: value ifFail: block
	"Equivalent to object basicAt: index put: value"
	^self ifFail: block indexedSlotOf: object at: index put: value! !

!VMMirror methodsFor: 'Object' stamp: 'ryan 6/8/2010 20:57'!
indexedSlotsOf: object "<Object> ^<Integer>"
	"Equivalent to victim basicSize."
	<primitive: 62>
	^0 "a fixed-size object has 0 indexed slots"! !

!VMMirror methodsFor: 'Object' stamp: 'ryan 7/28/2011 11:45'!
instaniateFixedClass: klass ifFail: failBlock
	| res |
	res := klass tryPrimitive: 70 withArgs: Array empty.
	res == ContextPart primitiveFailToken ifTrue: [^failBlock value].
	^res! !

!VMMirror methodsFor: 'Object' stamp: 'ryan 9/9/2010 15:30'!
instaniateVariableClass: klass withSize: smallInteger ifFail: failBlock
	^self ifFail: failBlock instaniateVariableClass: klass withSize: smallInteger! !

!VMMirror methodsFor: 'Object' stamp: 'Unknown 1/13/2013 12:15'!
instantiateFixedClass: klass
	<primitive: 70>
	klass isVariable ifTrue: [^self error: 'Variable class'].
	OutOfMemory signal.! !

!VMMirror methodsFor: 'Object' stamp: 'Unknown 1/13/2013 12:12'!
instantiateFixedClass: klass ifFail: failBlock
	"| res |
	res := klass tryPrimitive: 70 withArgs: Array empty.
	res == ContextPart primitiveFailToken ifTrue: [^failBlock value].
	^res"
	^self ifFail: failBlock instantiateFixedClass: klass! !

!VMMirror methodsFor: 'Object' stamp: 'Unknown 1/13/2013 12:16'!
instantiateVariableClass: klass withSize: smallInteger
	<primitive: 71>
	klass isVariable ifFalse: [^self error: 'Non-variable class'].
	(smallInteger isInteger and: [smallInteger >= 0])
		ifTrue: [OutOfMemory signal]
		ifFalse: [self error: 'Size must be positive integer'].! !

!VMMirror methodsFor: 'Object' stamp: 'eem 9/14/2012 15:43'!
instantiateVariableClass: klass withSize: smallInteger ifFail: failBlock
	^self ifFail: failBlock instantiateVariableClass: klass withSize: smallInteger! !

!VMMirror methodsFor: 'SmallInteger' stamp: 'ryan 9/9/2010 15:42'!
is: a equalTo: b ifFail: blk
	^self ifFail: blk is: a equalTo: b! !

!VMMirror methodsFor: 'SmallInteger' stamp: 'ryan 9/9/2010 15:42'!
is: a greaterOrEqual: b ifFail: blk
	^self ifFail: blk is: a greaterOrEqual: b! !

!VMMirror methodsFor: 'SmallInteger' stamp: 'ryan 9/9/2010 15:42'!
is: a greaterThan: b ifFail: blk
	^self ifFail: blk is: a greaterThan: b! !

!VMMirror methodsFor: 'Object' stamp: 'ryan 1/18/2010 00:00'!
is: a identicalTo: b
	<primitive: 110>! !

!VMMirror methodsFor: 'SmallInteger' stamp: 'ryan 9/9/2010 15:43'!
is: a lessOrEqual: b ifFail: blk
	^self ifFail: blk is: a lessOrEqual: b! !

!VMMirror methodsFor: 'SmallInteger' stamp: 'ryan 9/9/2010 15:43'!
is: a lessThan: b ifFail: blk
	^self ifFail: blk is: a lessThan: b! !

!VMMirror methodsFor: 'SmallInteger' stamp: 'ryan 9/9/2010 15:43'!
is: a notEqualTo: b ifFail: blk
	^self ifFail: blk is: a notEqualTo: b! !

!VMMirror methodsFor: 'as yet unclassified' stamp: 'ryan 3/5/2010 22:59'!
isUnix
	^OSProcess isUnix! !

!VMMirror methodsFor: 'as yet unclassified' stamp: 'ryan 3/5/2010 22:59'!
isUnixMac
	^OSProcess isUnixMac! !

!VMMirror methodsFor: 'as yet unclassified' stamp: 'ryan 3/5/2010 22:58'!
isWindows
	^OSProcess isWindows! !

!VMMirror methodsFor: 'FFI' stamp: 'ryan 10/19/2010 14:54'!
loadLibrary: libraryName "<String> ^<Integer>" ifFail: blk
	"Attempt to load the shared library/DLL named libraryName and
	 answer a handle which can be used in primFindSymbol:inLibrary:,
	 or fail if libraryName is invalid or libraryName cannot be found."
	^self ifFail: blk loadLibrary: libraryName! !

!VMMirror methodsFor: 'FFI' stamp: 'ryan 10/19/2010 14:42'!
malloc: byteSize "<Integer> ^<Integer>" ifFail: blk
	^self ifFail: blk malloc: byteSize! !

!VMMirror methodsFor: 'as yet unclassified' stamp: 'ryan 3/6/2010 16:30'!
millisecondClock
	<primitive: 135>! !

!VMMirror methodsFor: 'SmallInteger' stamp: 'ryan 9/9/2010 15:44'!
mod: a "SmallInteger" and: b "SmallInteger" ifFail: blk
	^self ifFail: blk mod: a and: b! !

!VMMirror methodsFor: 'SmallInteger' stamp: 'ryan 9/9/2010 15:44'!
multiply: a "SmallInteger" and: b "SmallInteger" ifFail: blk
	^self ifFail: blk multiply: a and: b! !

!VMMirror methodsFor: 'Object' stamp: 'ryan 9/9/2010 15:40'!
namedSlotOf: object at: index ifFail: block
	"Equivalent to object instVarAt: index"
	^self ifFail: block namedSlotOf: object at: index! !

!VMMirror methodsFor: 'Object' stamp: 'ryan 9/9/2010 15:41'!
namedSlotOf: object at: index put: value ifFail: block
	"Equivalent to object instVarAt: index put: value"
	^self ifFail: block namedSlotOf: object at: index put: value! !

!VMMirror methodsFor: 'Object' stamp: 'ryan 11/10/2010 10:45'!
object: o executeMethod: m with: args ifFail: blk
	^self ifFail: blk object: o with: args executeMethod: m ! !

!VMMirror methodsFor: 'Object' stamp: 'ryan 1/17/2010 23:56'!
quit
	<primitive: 113>! !

!VMMirror methodsFor: 'SmallInteger' stamp: 'ryan 9/9/2010 15:44'!
quo: a "SmallInteger" and: b "SmallInteger" ifFail: blk
	^self ifFail: blk quo: a and: b! !

!VMMirror methodsFor: 'SmallInteger' stamp: 'ryan 9/9/2010 15:45'!
subtract: a "SmallInteger" and: b "SmallInteger" ifFail: blk
	^self ifFail: blk subtract: a and: b! !

!SortedCollection methodsFor: '*NewspeakCore' stamp: 'GB 12/15/2010 13:15'!
indexFor: el "<E> ^ <Integer>"
	^self indexForInserting: el! !

!ByteArray methodsFor: '*NewspeakCore' stamp: 'GB 11/12/2008 20:17'!
copyWithSize: sz

| kopy |
	kopy := self class new: sz.
      1 to: (self size min: sz) do:[:i | kopy at: i put: (self at: i)].
	self size + 1 to: sz do:[: j | kopy at: j put: 0].
	^kopy
! !

!ByteString methodsFor: '*NewspeakCore' stamp: 'GB 11/12/2008 20:24'!
copyWithSize: sz

| ba |
	ba := self asByteArray copyWithSize: sz.
	^self class fromByteArray: ba.

! !

!Collection methodsFor: '*NewspeakCore' stamp: 'ryan 10/31/2009 18:41'!
asMutableArrayList
	^self asOrderedCollection! !

!Collection methodsFor: '*NewspeakCore' stamp: 'GB 11/10/2009 16:23'!
asSortedList
	"Answer a SortedCollection whose elements are the elements of the 
	receiver. The sort order is the default less than or equal."

	^ self asSortedCollection! !

!Collection methodsFor: '*NewspeakCore' stamp: 'GB 11/15/2009 03:20'!
asSortedList: block
	^self asSortedCollection: block! !

!Collection methodsFor: '*NewspeakCore' stamp: 'GB 1/8/2010 11:10'!
concatenate: aCollection

	^self , aCollection! !

!Behavior methodsFor: '*NewspeakCore' stamp: 'Ahe 9/5/2008 11:31'!
initializeSlotNamed: slotName "<String>" to: obj "<Object>"
	"Initialize the given slot of all instances of the reciever and its subclasses to the given object.
	
	Discussion: This method is useful when adding an import to a Newspeak class with existing instances.
	Warning: This is a transitory convenience method and will go away when the Newspeak IDE transparently takes care of initializing added slots."
	| ou |
	ou := ImplementationBase vmMirror objectUtil.
	self allSubInstancesDo: [ :instance |
		ou slotNamed: slotName of: instance put: obj]! !

!SequenceableCollection methodsFor: '*NewspeakCore' stamp: 'GB 11/12/2008 20:30'!
copyWithSize: sz 
	"Answer a copy of the receiver that is 1 bigger than the receiver and has 
	newElement at the last element."

	| newIC |
	newIC := self species new: sz.
	newIC 
		replaceFrom: 1
		to: self size
		with: self
		startingAt: 1.
	^newIC! !

Error subclass: #NewspeakError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'NewspeakCore'!

!NewspeakError class methodsFor: 'as yet unclassified' stamp: 'ryan 6/3/2011 09:41'!
messageNotUnderstoood: message "<Message>" receiver: receiver "<Object>^<MessageNotUnderstood>"
	^MessageNotUnderstood new
		message: message;
		receiver: receiver! !

!NewspeakError class methodsFor: 'as yet unclassified' stamp: 'ryan 6/3/2011 09:41'!
signalAssertionFailure: description "<String> ^<AssertionFailure>"
	^AssertionFailure signal: description! !

!NewspeakError class methodsFor: 'as yet unclassified' stamp: 'ryan 6/3/2011 09:42'!
signalPrimitiveFailed "^<Error>"
	^self signalPrimitiveFailed: 'Primitive failed'! !

!NewspeakError class methodsFor: 'as yet unclassified' stamp: 'ryan 6/3/2011 09:42'!
signalPrimitiveFailed: description "<String> ^<Error>"
	^self signal: description! !

!NewspeakError class methodsFor: 'as yet unclassified' stamp: 'ryan 6/3/2011 09:42'!
signalUninstantiable: description "<String> ^<Error>"
	^Error signal: description! !

!Array methodsFor: '*NewspeakCore' stamp: 'YM 11/24/2008 10:45'!
binarySearchFor: el between: start and: end toCompare: compare 
	"This does a binary search for the index such that if el was inserted before it
	  the receiver would remain sorted.  The receiver must be sorted relative to the
	  comparison block.  The comparison block should return true if the first block argument
	  cannot appear after the second block argument"

	| low high |
	low := start.
	high := end.
	[ low <= high ]
		whileTrue:
				[	| mid |
					mid := (low + high) // 2.
	  				(compare value: (self at: mid) value: el)
						ifTrue: [ low := mid + 1 ]	
						ifFalse: [ high := mid - 1 ].
				].
	^low! !

!Array methodsFor: '*NewspeakCore' stamp: 'YM 11/24/2008 11:19'!
copyWithSize: s
	| safeself |
	safeself := self.
	^(Array new: s) replaceFrom: 1 to: (s min: self size) with: safeself! !

!Array methodsFor: '*NewspeakCore' stamp: 'GB 11/20/2008 20:01'!
defaultSort: from to: to
	^self mergeSortFrom: from to: to by: [:a :b | a <= b]
! !

!Array methodsFor: '*NewspeakCore' stamp: 'YM 11/24/2008 11:00'!
putNilAt: index
	self at: index put: nil! !

!Array methodsFor: '*NewspeakCore' stamp: 'GB 11/20/2008 19:59'!
sort: from to: to  using: sortBlock
	^self mergeSortFrom: from to: to by: sortBlock! !

!Array methodsFor: '*NewspeakCore' stamp: 'GB 11/20/2008 20:02'!
sortUsing: sortBlock
	^self sort: sortBlock! !
ImplementationBase initialize!
NewspeakObject initialize!
